// Currency Exchange Extractor component dedicated for getting currency exchage from cbr.ru.
// Ver. 1.00 (20.09.2002) 
// Copyright (c) 2002 by Andrew A. Fedorov
// Information about updates of this component: http://www.bizkit.ru
// E-mail: fedorov@warlib.ru

unit CurrencyExchange;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, ComObj, Dialogs;

const DateFormat='dd''/''mm''/''yyyy';  

Type TCur=record
  Date:string;
  CurID:string;
  NumCode:string;
  CharCode:string;
  Nominal:string;
  Name:string;
  Value:string;
end;

Type TCurValues=array of TCur;

type
  TCurrencyExchange = class(TComponent)
  private
    { Private declarations }
    FBeginDate,FEndDate:TDate;
    FStrBeginDate,FStrEndDate:string;
    FCurID:TStringList;
    xmldoc,NodeList,XDate,xmlNode,node_attr:Variant;
    url_request,FXMLFileName:string;
    FValues,FConvert:TCurValues;
    FSilentMode:boolean;
    FCount,FCvtCount:integer;
    FIsConverted:boolean;
    FCurList:TStringList;
    FCurrency:string;

    FOnDownloadError: TNotifyEvent; //Error happend when downloading fault
    FOnCreateMSXMLError: TNotifyEvent; //Error happend when downloading fault
    FOnDataLoadingError: TNotifyEvent;
  protected
    procedure GetCurExchForTheDate;
    procedure GetCurExchForDatumRanges;
    procedure Convert;
    procedure GetCurrencies;
    { Protected declarations }
  public
    procedure GetData; //   rbc.ru
    function  GetCurrencyExchange:TCurValues; //    
    function GetIDByNumCode(const num:string):string;
    function GetIDByCharCode(const CharCode:string):string;
    function GetNumCodeByID(const ID:string):string;
    function GetCharCodeByID(const ID:string):string;
    function GetNameByID(const ID:string):string;

    constructor Create(AOwner: TComponent); override;
    destructor Destroy;
    { Public declarations }
  published
    property BeginDate:TDate read FBeginDate write FBeginDate; //        
    property EndDate:TDate read FEndDate write FEndDate;       //        
    property Currency:string read FCurrency write FCurrency; //ID      
    property CurID:TStringList read FCurID write FCurID; // ID    
    property SilentMode:boolean read FSilentMode write FSilentMode default true; //Silent mode. Generate error event insted of MessageBoxes
    property Count:integer read FCount; //Number of record
    property XMLFileName:string read FXMLFileName write FXMLFileName; //Full path to XML data file.     
    property CurrenciesList:TStringList read FCurList; //   ,     
    property OnDownloadError: TNotifyEvent read FOnDownloadError write FOnDownloadError;  //      ,      
    property OnCreateMSXMLError: TNotifyEvent read FOnCreateMSXMLError write FOnCreateMSXMLError;  //      ,      
    property OnDataLoadingError: TNotifyEvent read FOnDataLoadingError write FOnDataLoadingError;  //      ,      
    { Published declarations }
  end;

procedure Register;

implementation

constructor TCurrencyExchange.Create(AOwner: TComponent);
begin
  xmldoc:=CreateOleObject('Msxml.DOMDocument');
  BeginDate:=Date;EndDate:=Date;
  FXMLFileName:='';
  FCurID:=TStringList.Create;FCurID.Clear;
  FCurList:=TStringList.Create;FCurList.Clear;
  GetCurrencies;
  inherited Create(AOwner);
end;

procedure TCurrencyExchange.GetCurrencies;
var j:integer;
begin
  try
  if not FIsConverted then Convert;
  for j:=0 to FCvtCount-1 do begin
    FCurList.Add(FConvert[j].CharCode);
  end;
  except

  end;
end;

destructor TCurrencyExchange.Destroy;
begin
  xmldoc:=null;
  FCurID.Free;
  FCurList.Free;
  inherited Destroy;
end;

function TCurrencyExchange.GetCurrencyExchange:TCurValues;
begin
  Result:=FValues;
end;

//Get ID by NumCode
function TCurrencyExchange.GetIDByNumCode(const num:string):string;
var j:integer;
begin
  if not FIsConverted then Convert;
  for j:=0 to FCvtCount-1 do begin
    if AnsiSameText(num,FConvert[j].NumCode) then begin
      Result:=FConvert[j].CurID;
      break;
    end;
  end;
end;

//Get ID by CharCode
function TCurrencyExchange.GetIDByCharCode(const CharCode:string):string;
var j:integer;
begin
  if not FIsConverted then Convert;
  Result:='';
  for j:=0 to FCvtCount-1 do begin
    if AnsiSameText(CharCode,FConvert[j].CharCode) then begin
      Result:=FConvert[j].CurID;
      break;
    end;
  end;
end;

//Get CharCode by ID 
function TCurrencyExchange.GetCharCodeByID(const ID:string):string;
var j:integer;
begin
  if not FIsConverted then Convert;
  Result:='';
  for j:=0 to FCvtCount-1 do begin
    if AnsiSameText(ID,FConvert[j].CurID) then begin
      Result:=FConvert[j].CharCode;
      break;
    end;
  end;
end;

//Get Name by ID 
function TCurrencyExchange.GetNameByID(const ID:string):string;
var j:integer;
begin
  if not FIsConverted then Convert;
  Result:='';
  for j:=0 to FCvtCount-1 do begin
    if AnsiSameText(ID,FConvert[j].CurID) then begin
      Result:=FConvert[j].Name;
      break;
    end;
  end;
end;

//Get NumCode by ID
function TCurrencyExchange.GetNumCodeByID(const ID:string):string;
var j:integer;
begin
  if not FIsConverted then Convert;
  Result:='';
  for j:=0 to FCvtCount-1 do begin
    if AnsiSameText(ID,FConvert[j].CurID) then begin
      Result:=FConvert[j].NumCode;
      break;
    end;
  end;
end;

//Load data for conversion
procedure TCurrencyExchange.Convert;
var j:integer;
begin
  FIsConverted:=false;
  if trim(FXMLFileName)='' then
    FXMLFileName:=extractfilepath(ParamStr(0))+'XML_daily.xml';
  if not (xmldoc.Load(FXMLFileName)=True) then begin
    if FSilentMode then
      if Assigned(FOnDataLoadingError) then
        FOnDataLoadingError(self)
      else
        MessageDlg('Can''t load data file to conversion',mtError,[mbOK],0);
      Exit;
    end;

  NodeList:=xmldoc.selectNodes('ValCurs');
  xmlNode:= NodeList.Item(0).cloneNode(True);
  node_attr:= xmlNode.Attributes(0);
  XDate:=node_attr.Value;

  NodeList:=xmldoc.selectNodes('*/Valute');
  FCvtCount:=NodeList.length;
  SetLength(FConvert,FCvtCount);
  for j:= 0 to FCvtCount-1 do begin
      xmlNode:=NodeList.Item(j).cloneNode(True);
      node_attr:=xmlNode.Attributes(0);
      FConvert[j].CurID    :=node_attr.Value;
      FConvert[j].NumCode  :=xmlNode.childNodes(0).Text;
      FConvert[j].CharCode :=xmlNode.childNodes(1).Text;
      FConvert[j].Nominal  :=xmlNode.childNodes(2).Text;
      FConvert[j].Name     :=xmlNode.childNodes(3).Text;
      FConvert[j].Value    :=xmlNode.childNodes(4).Text;
  end;
  FIsConverted:=true;
end;

//Procedure return all currency exchane for BeginDate
procedure TCurrencyExchange.GetCurExchForTheDate;
var iIndex,j:integer;
begin
  url_request:='http://www.cbr.ru/scripts/XML_daily.asp?date_req=' + Trim(FStrBeginDate);
  if not (xmldoc.Load(url_request)=True) then begin
    if FSilentMode then
      if Assigned(FOnDownloadError) then
        FOnDownloadError(self)
      else
        MessageDlg('Can''t get currency excange data from ''Central Bank of Russia''',mtError,[mbOK],0);
      Exit;
    end;

  NodeList:=xmldoc.selectNodes('ValCurs');
  xmlNode:= NodeList.Item(0).cloneNode(True);
  node_attr:= xmlNode.Attributes(0);
  XDate:=node_attr.Value;

  NodeList:=xmldoc.selectNodes('*/Valute');
  for iIndex:= 0 to NodeList.length - 1 do begin
      xmlNode:=NodeList.Item(iIndex).cloneNode(True);
      SetLength(FValues,FCurID.Count);
      FCount:=FCurID.Count;
      if FCount>0 then
        for j:=0 to FCount-1 do begin
          node_attr:=xmlNode.Attributes(0);
          if node_attr.Value = Trim(FCurID[j]) then begin
            FValues[j].Date    :=FStrBeginDate;
            FValues[j].CurID    :=node_attr.Value;
            FValues[j].NumCode  :=xmlNode.childNodes(0).Text;
            FValues[j].CharCode :=xmlNode.childNodes(1).Text;
            FValues[j].Nominal  :=xmlNode.childNodes(2).Text;
            FValues[j].Name     :=xmlNode.childNodes(3).Text;
            FValues[j].Value    :=xmlNode.childNodes(4).Text;
          end;
        end;
  end;
end;

//Get currency exchange for specified currency for datum ranges 
procedure TCurrencyExchange.GetCurExchForDatumRanges;
var j:integer;
//    CurrencyID:string;
    tmpNumCode,tmpCharCode,tmpName:string;
begin
//  CurrencyID:=Trim(FCurID[0]);
  url_request:='http://www.cbr.ru/scripts/XML_dynamic.asp?date_req1=' + FStrBeginDate+'&date_req2=' + FStrEndDate+'&VAL_NM_RQ='+FCurrency;
  if not (xmldoc.Load(url_request)=True) then begin
    if FSilentMode then
      if Assigned(FOnDownloadError) then
        FOnDownloadError(self)
      else
        MessageDlg('Can''t get currency excange data from ''Central Bank of Russia''',mtError,[mbOK],0);
      Exit;
    end;

  NodeList:=xmldoc.selectNodes('*/Record');
  FCount:=integer(NodeList.length);
  SetLength(FValues,FCount);
  tmpNumCode:=GetNumCodeByID(FCurrency);
  tmpCharCode:=GetCharCodeByID(FCurrency);
  tmpName:=GetNameByID(FCurrency);
  for j:= 0 to FCount - 1 do begin
      xmlNode:=NodeList.Item(j).cloneNode(True);
      node_attr:=xmlNode.Attributes(0);
      FValues[j].Date     :=node_attr.Value;
      FValues[j].CurID    :=FCurrency;
      FValues[j].Nominal  :=xmlNode.childNodes(0).Text;
      FValues[j].Value    :=xmlNode.childNodes(1).Text;
      FValues[j].NumCode  :=tmpNumCode;
      FValues[j].CharCode :=tmpCharCode;
      FValues[j].Name     :=tmpName;
  end;
end;

//Procedure inintiate connection with rbc.ru and get currency exchange from it.
procedure TCurrencyExchange.GetData;
var TmpDateSeparator:char;
begin
  if not (VarType(xmldoc)=VarDispatch) then begin
    if FSilentMode then
      if Assigned(FOnCreateMSXMLError) then
        FOnCreateMSXMLError(self)
      else
        MessageDlg('Can''t create MSXML object',mtError,[mbOK],0);
      exit;
  end;

  xmldoc.async:=False;
//  TmpDateSeparator:=DateSeparator;
//  DateSeparator:='/';
  FStrBeginDate:=FormatDateTime(DateFormat,FBeginDate);
  FStrEndDate:=FormatDateTime(DateFormat,FEndDate);
//  DateSeparator:=TmpDateSeparator;

  if FEndDate>now then FEndDate:=now;

  if (FBeginDate=FEndDate) then begin
    GetCurExchForTheDate;
  end else
    GetCurExchForDatumRanges;
    
end;

procedure Register;
begin
  RegisterComponents('BizKit', [TCurrencyExchange]);
end;

end.
 